command -nargs=1 -complete=dir DlcPro call s:ProjectSet('dlcpro-new', '<args>')
command -nargs=1 -complete=dir DlcProOld call s:ProjectSet('dlcpro', '<args>')
command -nargs=1 -complete=dir DlcProShg call s:ProjectSet('shg', '<args>')
command -nargs=1 -complete=dir DlcProGui call s:ProjectSet('dlcpro-gui', '<args>')
command -nargs=1 -complete=dir DlcProTui call s:ProjectSet('dlcpro-tui', '<args>')
command -nargs=1 -complete=dir DlcProTuiSimulator call s:ProjectSet('dlcpro-tui-simulator', '<args>')
command -nargs=1 -complete=dir DlcProCan call s:ProjectSet('dlcpro-can', '<args>')
command -nargs=1 -complete=dir DlcProSpecalyser call s:ProjectSet('dlcpro-specalyser', '<args>')
command -nargs=1 -complete=dir Topmode call s:ProjectSet('topmode', '<args>')
command -nargs=1 -complete=dir TopmodeGui call s:ProjectSet('topmode-gui', '<args>')
command -nargs=1 -complete=dir DigiFalc call s:ProjectSet('digifalc', '<args>')
command -nargs=1 -complete=dir ServoBoard call s:ProjectSet('servoboard', '<args>')
command -nargs=1 -complete=dir DlMotor call s:ProjectSet('dl-motor', '<args>')
command -nargs=1 -complete=dir Pfd call s:ProjectSet('pfd', '<args>')
command -nargs=1 -complete=dir OperationPanelF1 call s:ProjectSet('operation-panel-f1', '<args>')
command -nargs=1 -complete=dir OperationPanelF4 call s:ProjectSet('operation-panel-f4', '<args>')
command -nargs=1 -complete=dir DeCoF call s:ProjectSet('decof', '<args>')
command DeviceFirmwareUpdate call s:DeviceFirmwareUpdate()
command -nargs=? -complete=file JenkinsLinter call s:Jenkins_linter('<args>')

let s:path_orig = $PATH
set titlestring=

function! s:ProjectSet(project_type, project_base_dir)
    let g:project_type = a:project_type
    let g:jenkins_url = 'http://jenkins.toptica.com'

    " directories
    if a:project_base_dir != ''
        if (isdirectory(fnamemodify(a:project_base_dir, ':p:h:h').'/src'))
            let s:ProjectBaseDir = fnamemodify(a:project_base_dir, ':p:h:h')
        elseif (isdirectory(fnamemodify(a:project_base_dir, ':p:h').'/src'))
            let s:ProjectBaseDir = fnamemodify(a:project_base_dir, ':p:h')
        else
            let s:ProjectBaseDir = fnamemodify(a:project_base_dir, ':p')
        endif
    else
        " defaults
        if (g:project_type == 'dlcpro-new')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware'
        elseif (g:project_type == 'dlcpro')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware'
        elseif (g:project_type == 'dlcpro-tui')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware'
        elseif (g:project_type == 'dlcpro-tui-simulator')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware.tui-simulator'
        elseif (g:project_type == 'dlcpro-can')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware'
        elseif (g:project_type == 'dlcpro-specalyser')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/firmware'
        elseif (g:project_type == 'shg')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/shg-firmware'
        elseif (g:project_type == 'dlcpro-gui')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/pc-gui'
        elseif (g:project_type == 'topmode')
            let s:ProjectBaseDir = '/home/stefan/topmode/firmware'
        elseif (g:project_type == 'topmode-gui')
            let s:ProjectBaseDir = '/home/stefan/topmode/pc-gui'
        elseif (g:project_type == 'digifalc')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/falc/firmware'
        elseif (g:project_type == 'servoboard')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/mta/firmware'
        elseif (g:project_type == 'dl-motor')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/dl-motor'
        elseif (g:project_type == 'pfd')
            let s:ProjectBaseDir = '/home/stefan/dlcpro/pfd/firmware'
        else
            echo "no project"
        endif
    endif

    let g:ProjectSrcDirRel = 'src'
    let s:ProjectSrcDir = s:ProjectBaseDir.'/'.g:ProjectSrcDirRel

    function! g:Compiler_version()
        if exists('g:ProjectBuildDir')
            let cmakefile = g:ProjectBuildDir . '/CMakeCache.txt'
            if filereadable(cmakefile)
                for line in readfile(cmakefile)
                    if match(line, 'NEW_COMPILER:BOOL=OFF') >= 0
                        return 'old'
                    elseif match(line, 'NEW_COMPILER:BOOL=ON') >= 0
                        return 'new'
                    endif
                endfor
            endif
        endif
        return 0
    endfunction

    " vim path
    execute 'cd '.s:ProjectSrcDir
    execute 'set path-=./**'
    execute 'set path+=' .  s:ProjectSrcDir.'/**'
    " execute 'set path+=' .  g:ProjectBuildDir.'/**'

    let s:invoke_options = '' " used for --project=...

    if (g:project_type == 'dlcpro-new') || (g:project_type == 'dlcpro')
        let s:invoke_options = ' --project='.g:project_type
        set wildignore-=**/firmware/src/device-control/**
        set wildignore+=**/shg-firmware/**
        set titlestring=%<%t\ (%{expand('%:p:h')})%=project:\ %{g:project_type}\ \ \ \ compiler:\ %{g:Compiler_version()}
        let g:DeviceIP = 'dlc_pro__040083'
        let g:DebugRemote = v:true
        if (g:project_type == 'dlcpro-new')
            let g:ProjectBuildDir = s:ProjectSrcDir.'/.build/dlcpro-new' " FIXME: get from ???
            let g:termdebugger = 'arm-v7a-linux-gnueabihf-gdb' " FIXME: get from ???
        else
            let g:ProjectBuildDir = s:ProjectSrcDir.'/.build/dlcpro' " FIXME: get from ???
            let g:termdebugger = 'arm-cortexa8-linux-gnueabi-gdb' " FIXME: get from ???
        endif
        let s:gdb_connect_script = g:ProjectBuildDir.'/gdb-connect.txt'
    elseif ((g:project_type == 'dlcpro-tui') || (g:project_type == 'dlcpro-tui-simulator'))
        let s:Program = '/user-interface/src/user-interface'
        let s:Elffile = s:Program
        let g:ProgramRemote = '/opt/app/bin/user-interface'
        set wildignore-=**/firmware/src/device-control/**
        set wildignore+=**/shg-firmware/**
        let g:DeviceIP = 'dlc_pro__040083'
        let g:DebugRemote = v:true
    elseif (g:project_type == 'dlcpro-can')
        let s:Program = '/canopen/can-updater'
        let s:Elffile = s:Program
        let g:ProgramRemote = '/opt/app/bin/can-updater'
        set wildignore-=**/firmware/src/device-control/**
        set wildignore+=**/shg-firmware/**
        let g:DeviceIP = 'dlc_pro__040083'
        let g:DebugRemote = v:true
    elseif (g:project_type == 'dlcpro-specalyser')
        let s:Program = '/specalyser/specalyser'
        let s:Elffile = s:Program
        let g:ProgramRemote = '/opt/app/bin/specalyser'
        set wildignore-=**/firmware/src/device-control/**
        set wildignore+=**/shg-firmware/**
        let g:DeviceIP = 'dlc_pro__040083'
        let g:DebugRemote = v:true
    elseif (g:project_type == 'shg')
        let s:Program = '/shg-firmware/device-control/device-control-shg'
        let s:Elffile = s:Program
        let g:ProgramRemote = '/opt/app/bin/device-control-shg'
        set wildignore-=**/shg-firmware/**
        set wildignore+=**/firmware/src/device-control/**
        let g:DeviceIP = 'dlc_pro__040083'
        let g:DebugRemote = v:true
        " let g:GdbPort = '6666'
        " let g:SshOpts = '-o ForwardAgent=yes -o ProxyCommand="ssh -o RemoteCommand=none -q -W shg:22 root@%h" -o StrictHostKeyChecking=no -o UserKnownHostsFile=/dev/null -o LogLevel=ERROR'
        " let g:SshOpts2 = "-L localhost:1998:localhost:1998 -L localhost:1999:localhost:1999"
    elseif (g:project_type == 'dlcpro-gui')
        let g:ProjectBuildDir = s:ProjectSrcDir.'/.build' " FIXME (for DeviceDebug)
        let s:Program = '/TOPAS_DLC_pro'
        let s:Elffile = s:Program
        let g:termdebugger = 'gdb'
        let g:DebugRemote = v:false
    elseif (g:project_type == 'topmode')
        let g:DeviceIP = 'topmode_stefan'
        let g:DebugRemote = v:true
    elseif (g:project_type == 'topmode-gui')
        let g:DebugRemote = v:false
    elseif (g:project_type == 'digifalc')
        let s:Program = '/digifalc-image.bin'
        let s:Elffile = '/application/digifalc.elf'
        let g:DebugRemote = v:true
        let g:termdebugger = 'arm-none-eabi-gdb'
    elseif (g:project_type == 'servoboard')
        let s:Program = '/servo-board-image.bin'
        let s:Elffile = '/application/servo-board.elf'
        let g:DebugRemote = v:true
        let g:termdebugger = 'arm-none-eabi-gdb'
    elseif (g:project_type == 'dl-motor')
        let s:Program = '/dl-motor-image.bin'
        let s:Elffile = '/application/dl-motor.elf'
        let g:DebugRemote = v:true
        let g:termdebugger = 'arm-none-eabi-gdb'
    elseif (g:project_type == 'pfd')
        let g:ProjectBuildDir = s:ProjectSrcDir.'/.build/target/firmware/Debug'
        let s:Program = '/pfd-firmware.bin'
        let s:Elffile = '/pfd-firmware.elf'
        let g:DebugRemote = v:true
        let g:termdebugger = 'arm-none-eabi-gdb'
        let s:gdb_connect_script = g:ProjectBuildDir.'/gdb-connect.txt'
    elseif (g:project_type == 'operation-panel-f1')
        let g:stm32_target = 'STM32F10X'
        let s:Program = '/firmware/operation_panel_'.g:stm32_target.'.bin'
        let s:Elffile = '/firmware/operation_panel_'.g:stm32_target.'.elf'
        " let s:makegoals = ['artifacts']
        let g:DebugRemote = v:true
        " let g:GdbPort = '3333'
        let g:termdebugger = 'arm-none-eabi-gdb'
        " let s:stm_cfg = 'stm32f1x'
    elseif (g:project_type == 'operation-panel-f4')
        let g:stm32_target = 'STM32F4XX'
        let s:Program = '/firmware/operation_panel_'.g:stm32_target.'.bin'
        let s:Elffile = '/firmware/operation_panel_'.g:stm32_target.'.elf'
        " let s:makegoals = ['artifacts']
        let g:DebugRemote = v:true
        " let g:GdbPort = '3333'
        let g:termdebugger = 'arm-none-eabi-gdb'
        " let s:stm_cfg = 'stm32f4x'
    elseif (g:project_type == 'decof')
        " let s:makegoals = ['all']
        let g:DebugRemote = v:false
    else
        echo "no project"
    endif

    " Settings for invoke development
    let s:invoke = 'invoke'
    let s:tasks_file_dev = '/home/stefan/tools/invoke/'.g:project_type.'/tasks.py'
    if filereadable(s:tasks_file_dev)
        call system('cp '.s:tasks_file_dev.' tasks.py') " copy development version to project
    elseif filereadable('tasks.py')
        " do nothing
    else
        echoerr 'tasks.py not found'
    endif

    " editor settings
    set spell spelllang=en,de
    set expandtab
    set cinoptions=l1,g2,h2,N-2,t0,+0,(0,w1,Ws,m1,)100,*100
    set textwidth=120

    " python tags
    execute "set tags+=" . s:ProjectBaseDir . '/tags'

    " compiler
    compiler gcc
    if exists("s:Program")
        let g:Program = g:ProjectBuildDir.s:Program
        let g:Elffile = g:ProjectBuildDir.s:Elffile
    endif
    command! -complete=customlist,GetAllMakeCompletions -nargs=* MakeCmd call s:Make('<args>', 'async')

    " configure quickfix window for asyncrun
    augroup QuickfixStatus
        autocmd BufWinEnter quickfix setlocal 
                    \ statusline=%t\ [%{g:asyncrun_status}]\ %{exists('w:quickfix_title')?\ '\ '.w:quickfix_title\ :\ ''}\ %=%-15(%l,%c%V%)\ %P
    augroup END

    function! s:DeviceFirmwareUpdate()
        call s:Make('kill', 'bang')
        call s:Make('flash', 'bang')
        call s:Make('run', 'bang')
    endfunction

    command! DeviceDebug call s:DeviceDebug(0)
    command! DeviceDebugAttach call s:DeviceDebug(1)

    " little helpers
    let $QT_FONT_DPI=96 " Size for qt-application in debugger


endfunction

" ====
" Make
" ====
function! GetAllMakeCompletions(ArgLead, CmdLine, CursorPos)
    let CmdList = split(a:CmdLine)
    let tasks = []
    let tasks += split(system(s:invoke.' --complete --'))
    if len(CmdList) > 1
        let last_task = CmdList[-1]
        if a:ArgLead[0] == '-' || a:ArgLead == '' && last_task[0] == '-'
            let last_task = CmdList[-2]
            let last_opt = CmdList[-1]
        elseif a:ArgLead != '' && a:ArgLead[0] != '-' && len(CmdList) > 2
            let last_task = CmdList[-3]
            let last_opt = CmdList[-2]
        endif
        " Expand option
        if count(tasks, last_task) > 0
            let tasks += split(system(s:invoke.' --complete -- '.last_task.' -'))
        endif
        if exists('last_opt')
            " Expand values of option
            let helptext = systemlist(s:invoke.' '.last_task.' --help')
            let i = match(helptext, last_opt)
            if i > 0
                let values = []
                call substitute(helptext[i], '"\([^"]\+\)"', '\=add(values, submatch(1))', 'g')
                if len(values) > 0
                    let tasks += values
                endif
            endif
        endif
    endif
    if a:ArgLead != ''
        " Filter option if started to type
        call filter(tasks, 'v:val =~ "^'. a:ArgLead .'"')
    endif
    return tasks
endfunction

function! s:Make(args, async_mode)
    call asyncrun#quickfix_toggle(10, 1)
    let l:makeprg = s:invoke.' -e'
    let l:args = a:args
    if l:args == ''
        let l:args = 'build' " default
    endif

    let task = split(l:args)[0]
    let options = system(s:invoke.' --complete -- '.task.' -')

    " Add --project if necessary
    if options =~ '--project' && '--project' !~ l:args
        let l:args .= ' --project='.g:project_type
    endif
    " Add --target-ip if necessary
    if options =~ '--target-ip' && '--target-ip' !~ l:args
        let l:args .= ' --target-ip='.g:DeviceIP
    endif

    let &makeprg = l:makeprg
    execute 'AsyncRun -mode='.a:async_mode.' -save=2 -program=make @ '.l:args
endfunction

function! s:Call_and_log(cmd)
    echom a:cmd
    let r = system(a:cmd)
    let e = v:shell_error
    if (e != 0)
        echom 'return value: '.e.', output: "'.r.'"'
    endif
    return v:shell_error
endfunction

function! g:JLinkSWOviewer()
    let cmd = '/JLinkSWOViewerCLExe -device STM32H743ZI -itmmask 0xffffffff -swofreq 450000'
    call asyncrun#quickfix_toggle(10, 1)
    execute 'AsyncRun -mode=async @ ' . cmd
endfunction

function! s:SendToDebugger(command)
    call term_sendkeys('', a:command . "\n")
endfunction

function! s:StartDebugger(elffile, attach)
    if (a:attach == 0)
        Termdebug
    else
        execute 'Termdebug ' . a:elffile
    endif
endfunction

let g:DlcproBasePath = "/jenkins/workspace/pro--firmware_release_1.9.0-DCESJ5C5R577IG5QFEWTML22UFDDZCJDGFLMDA4DCD3V2ZAGVEJA/source/"
function! s:DeviceDebug(attach)
    if g:DebugRemote
        if 'dlcpro' =~ g:project_type " FIXME
            call system(s:invoke.' -e kill '.g:DeviceIP)
        endif
        autocmd! User TermdebugStartPre
        autocmd! User TermdebugStartPost
        autocmd! User TermdebugStopPost
        let gdb_server_cmd = s:invoke.' -e gdb-server'
        if exists('g:DeviceIP')
            let gdb_server_cmd .= ' --target-ip='.g:DeviceIP
        endif
        let gdb_server_cmd .= s:invoke_options
        if v:true
            call system(gdb_server_cmd.'&')
        else
            call asyncrun#quickfix_toggle(3, 1)
            execute 'AsyncRun '.gdb_server_cmd
            autocmd User TermdebugStopPost execute 'AsyncStop'
        endif
        sleep 1
        if exists('s:gdb_connect_script')
            autocmd User TermdebugStartPost call term_sendkeys('',"source ".s:gdb_connect_script."\n")
        endif
        Termdebug
        call s:SendToDebugger('set confirm off')
        " Close Program window
        execute '2hide'
    else
        call s:StartDebugger(g:Elffile, a:attach)
        call s:SendToDebugger('file '.g:Elffile)
        call s:SendToDebugger('break main')
        call s:SendToDebugger('run')
    endif
endfunction

" ================
" Regression Tests
" ================
function! g:DlcproRegtestCmd(arguments)
    return s:DlcproRegtestCmd('',              '',            '0', '1', '',          '--capture=no',                  a:arguments)
endfunction
function! g:DlcproRegtestDlProCmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro_proto_050003', 'elab-dlcpro2', '2', '1', 'DLpro',     '',                              a:arguments)
endfunction
function! g:DlcproRegtestTaProCmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro_045396',  'elab-dlcpro2', '3', '1', 'TApro',     '-m "not usb and not usbstick"', a:arguments)
endfunction
function! g:DlcproRegtestCtlCmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro__040120', 'elab-dlcpro2', '1', '1', 'CTL',       '-m "not usb and not usbstick"', a:arguments)
endfunction
function! g:DlcproRegtestDualDlCmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro_041299', 'elab-dlcpro2', '4', '2', 'DLpro',     '-m "not usb and not usbstick"', a:arguments)
endfunction
function! g:DlcproRegtestDualDl1Cmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro_041299', 'elab-dlcpro2', '4', '1', 'DLpro',     '-m "not usb and not usbstick"', a:arguments)
endfunction
function! g:DlcproRegtestShgProCmd(arguments)
    return s:DlcproRegtestCmd('dlc_pro_041355', 'elab-dlcpro2', '7', '1', 'TA-SHGpro', '-m "not usb and not usbstick"', a:arguments)
endfunction

command -nargs=1 -complete=file DlcproRegtest        call s:DlcproRegtest(g:DlcproRegtestCmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestDlPro   call s:DlcproRegtest(g:DlcproRegtestDlProCmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestTaPro   call s:DlcproRegtest(g:DlcproRegtestTaProCmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestCtl     call s:DlcproRegtest(g:DlcproRegtestCtlCmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestDualDl  call s:DlcproRegtest(g:DlcproRegtestDualDl1Cmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestDualDl1 call s:DlcproRegtest(g:DlcproRegtestDualDl1Cmd('<args>'))
command -nargs=1 -complete=file DlcproRegtestShgPro  call s:DlcproRegtest(g:DlcproRegtestShgProCmd('<args>'))

let g:DlcproRegtest_fast_restart = 1
let g:DlcproRegtest_marks = '"-m (no_crash_without_mc_fpga and not usb and not usbstick and not si and not si1 and not servo_control and not eom and not cavity and not cell_spectroscopy and not falc and not pfd and not smc)"'
"let g:DlcproRegtest_marks = ""
let g:DlcproRegtest_lasertype = "DLpro"
let g:DlcproRegtest_powerswitch_ip = ""

function! s:DlcproRegtestCmd(ip, powerswitch_ip, powerplug, laser_count, laser_type, opts, arguments)
    if (a:ip == '')
        let ip = g:DeviceIP
    else
        let ip = a:ip
    endif
    if (a:laser_type == '')
        let laser_type = g:DlcproRegtest_lasertype
    else
        let laser_type = a:laser_type
    endif
    if (a:powerswitch_ip == '')
        let powerswitch_ip = g:DlcproRegtest_powerswitch_ip
    else
        let powerswitch_ip = a:powerswitch_ip
    endif

    let archive_dir = g:ProjectBuildDir."/artifacts"
    let license_builddir = s:ProjectBaseDir.'/build.license'
    let licensetool = license_builddir."/dlcprolicense-tool/dlcprolicense-tool"
    let $JENKINS_URL = g:jenkins_url

    let test_cmd =
                \s:ProjectSrcDir."/test/python-env/bin/python -u -m pytest ".
                \"--showlocals --tb=long --verbose --cache-clear ".
                \"-o junit_family=xunit1 ".
                \"--junit-xml=regtest.".laser_type.".xml ".
                \"--debug_build ".
                \"--laser_count=".a:laser_count." ".
                \"--laser1_type=".laser_type." ".
                \"--log-file=regtest.".laser_type.".log ".
                \"--target_ip=".ip." ".
                \"--powerswitch_ip=".powerswitch_ip." ".
                \"--powerswitch_passwd=nimda ".
                \"--power_plug=".a:powerplug." ".
                \"--power_plug_fan=8"." ".
                \"--license_tool=".licensetool." ".
                \"--license_keyfile=".s:ProjectSrcDir."/license/libdlcprolicense/rsa-private.key ".
                \"--log-cli-level=INFO "
                \"--log-file-level=DEBUG "
                \""

    if (g:DlcproRegtest_fast_restart == 1)
        let test_cmd .= "--fast_restart "
    endif

    " hint: --collect-only

    let test_cmd .= a:opts." ". g:DlcproRegtest_marks." ".a:arguments
    let @+ = test_cmd
    return test_cmd
endfunction

function! s:DlcproRegtest(test_cmd)
    execute "wa"

    " Build license tool
    let license_builddir = s:ProjectBaseDir.'/build.license'
    let licensetool = license_builddir."/libdlcprolicense/dlcprolicense-tool"
    let license_cmake = "cmake -DLICENSE_TOOL=1 -DCMAKE_BUILD_TYPE=Release ".s:ProjectSrcDir."/license"
    let license_make = "make dlcprolicense-tool"
    if !executable(licensetool)
        call mkdir(license_builddir, "p")
        call term_start(license_cmake, {'cwd' : license_builddir})
        sleep 2
        call term_start(license_make, {'cwd' : license_builddir})
        sleep 5
    endif

    " Build command
    echom a:test_cmd

    " Execute pytest
    execute "terminal ++shell cd " s:ProjectSrcDir. "/test && " . a:test_cmd
"    call term_start(a:test_cmd, {'cwd' : s:ProjectSrcDir."/test"})
endfunction

" ------
" Pandoc
" ------
command TopticaBeamer execute('!pandoc -s -t beamer -H toptica-style.tex -o %:r.pdf %')

" -----------------
" Jenkins validator
" -----------------
function! s:Jenkins_linter(jenkinsfile)
    if a:jenkinsfile == ''
        let jenkinsfile = expand("%")
    else
        let jenkinsfile = a:jenkinsfile
    endif

    if !filereadable(jenkinsfile)
        echoerr("File not found: ".jenkinsfile)
        return
    endif

    if !exists("s:user")
        let s:user = input("jenkins-user:")
    endif
    if !exists("s:passwd")
        let s:passwd = inputsecret("password:")
        echo "\n"
    endif
    let user_pass = s:user.":".s:passwd

    let l:out = system("curl --no-progress-meter --user '".user_pass."' -X POST -F 'jenkinsfile=<".jenkinsfile."' ".g:jenkins_url."/pipeline-model-converter/validate")
    if match(l:out, "HTTP ERROR 401") >= 0
        " Forget credentials on authorization error
        unlet s:user
        unlet s:passwd
    endif
    echo l:out
endfunction
